import { Injectable } from '@angular/core';
import { HttpClientModule, HttpClient, HttpHeaders, HttpErrorResponse } from '@angular/common/http';
import { Observable, Subject, BehaviorSubject } from 'rxjs';
import { catchError, map, tap, take,filter } from 'rxjs/operators';
import { ActivatedRoute, Router } from '@angular/router';
import {LocationStrategy, Location} from '@angular/common';
// import { keypair } from 'keypair';
import * as keypair from 'keypair';
import * as forge from "node-forge";
import { Identity, AuthToken, KeyCert, SshAuthzServer } from './identity';
import {BackendSelectionService} from './backend-selection.service';
import { throwError } from 'rxjs';


export class SshauthzServer {}

@Injectable({
  providedIn: 'root'
})
export class AuthorisationService {
  // public token: BehaviorSubject<AuthToken>;
  public token: BehaviorSubject<AuthToken>;
  public sshAuthzServers: BehaviorSubject<SshAuthzServer[]>;
  public loggedInAuthZ: BehaviorSubject<SshAuthzServer[]>;
  public loggedOutAuthZ: BehaviorSubject<SshAuthzServer[]>;
  public readyToNavigate: Subject<[Boolean,string]>;
  public progress: Subject<string>;
  // private keyCert: Subject<KeyCert>;
  public backendURI: string;
  public statusMsg: BehaviorSubject<string>;
  public agentContents: BehaviorSubject<any>;
  public keys: KeyCert[];


  constructor(private http: HttpClient,
              private locationStrategy: LocationStrategy,
              private route: ActivatedRoute,
              private router: Router,
              private backendSelectionService: BackendSelectionService,
              private location: Location) {
                this.token = new BehaviorSubject<AuthToken>(null);
                this.readyToNavigate = new Subject<[Boolean,string]>();
                this.readyToNavigate.next([false,'']);
                this.progress = new Subject<string>();
                this.progress.next("");
                // this.token.subscribe(token => this.getCert(token));
                this.route.fragment.subscribe(frag => this.storeToken(frag));
                this.sshAuthzServers = new BehaviorSubject<SshAuthzServer[]>([]);
                this.loggedInAuthZ = new BehaviorSubject<SshAuthzServer[]>([]);
                this.loggedOutAuthZ = new BehaviorSubject<SshAuthzServer[]>([]);
                this.backendURI = null;
                this.agentContents = new BehaviorSubject([]);
                this.keys = [];
                this.getSshAuthzServers();
                this.keys = [];
    this.backendSelectionService.apiserver.pipe(filter((v) => v !== null && v !== undefined))
      .subscribe((value) => { this.backendURI = value.tes ; this.updateAgentContents() })
    this.agentContents.subscribe((value) => this.updateLoggedAuthZ());
    this.sshAuthzServers.subscribe((value) => this.updateLoggedAuthZ());
 }

 public setStatusMsg(statusMsg: BehaviorSubject<any>) {
   this.statusMsg = statusMsg;
 }

 storeLocalAuthZ(authz: any) {
   try {
     localStorage.setItem('localauthservers',JSON.stringify(authz));
   } catch {
   }
   this.getSshAuthzServers();
 }
 removeLocalAuthZ() {
   localStorage.removeItem('localauthservers');
   this.getSshAuthzServers();
 }

 getSshAuthzServers() {
   let headers = new HttpHeaders();
   let options = { headers: headers, withCredentials: false};
   this.http.get<SshAuthzServer[]>('./assets/config/authservers.json',options)
                    .pipe(catchError(this.handleError('getSshAuthzServers')))
                    .subscribe(resp => this.updateSshAuthzServers(resp));
 }

 private updateLoggedAuthZ() {
     let agentContents = this.agentContents.value
     let authzServers = this.sshAuthzServers.value
     let loggedin = []
     let loggedout = []
     var found: boolean;
     for (let s of authzServers) {
         found=false;
         for (let cert of agentContents) {
           if ('Signing CA' in cert) {
             for (let ca of cert['Signing CA']) {
               if (ca == s.cafp) {
                 loggedin.push(s)
                 found=true;
                 continue;
               }
             }
           }
           if (found) {
               continue;
           }
         }
         if (!found)  {
           loggedout.push(s)
         }
     }
     this.loggedOutAuthZ.next(loggedout);
     this.loggedInAuthZ.next(loggedin);
 }
        

 updateSshAuthzServers(resp) {
   var auths: SshAuthzServer[];
   var localauths: SshAuthzServer[] = [];
   var server: SshAuthzServer;
   auths = <SshAuthzServer[]>resp;
   try {
       localauths = JSON.parse(localStorage.getItem('localcomputesites'))
   } catch {
       localauths = []
   }
   if (localauths !== null) {
       for (server of localauths) {
           auths.push(server);
       }
   }
   this.sshAuthzServers.next(auths);
 }

 storeToken(frag: string) {
   if (frag === undefined || frag == null) {
       return;
   }
   let tokenmatch = null;
   let statematch = null;
   if (!(frag === undefined) && !(frag == null)) {
     tokenmatch = frag.match(/access_token\=([^&]+)(&|$)/);
     statematch = frag.match(/state\=([^&]+)(&|$)/);
   }
   if (tokenmatch == null || statematch == null) {
     return;
   }

   let accesstoken = tokenmatch[1];
   let state = statematch[1];
   // this.router.navigate(['/']);

   //Verify that the state matched the nonce we used when initiating login
   let tuple = JSON.parse(localStorage.getItem('authservice'));
   if (tuple[1] != state) {
     return
   }

   this.token.next(new AuthToken(tokenmatch[1],tuple[0]));
   //this.token = new AuthToken(tokenmatch[1],tuple[0]);


 }

 public getCert() {
   var token: AuthToken;
   token = this.token.value
   if (this.backendSelectionService.apiserver.value === null) {
     this.backendSelectionService.apiserver.pipe(take(1)).subscribe(() => this.getCert())
   }
   if (token.token === undefined || token.token === '' || token.token == null) {
     this.token.pipe(take(1)).subscribe(() => this.getCert())
   }


   let starttime = new Date();
   let newkeypair = keypair();
   let publicKey = forge.pki.publicKeyFromPem(newkeypair.public);
   let sshpub = forge.ssh.publicKeyToOpenSSH(publicKey, 'sv2@monash.edu');
   let endtime = new Date();

   let headers = new HttpHeaders();
   let options = { headers: headers, withCredentials: true};

   let data = {'token': token.token, 'pubkey': sshpub, 'signing_url': token.sshauthzservice.sign};

   this.http.post<any>(this.backendURI+'/getcert',data, options)
                      .pipe(catchError(this.handleError([])))
                      .subscribe(resp => this.makeKeyCert(newkeypair.private, resp, token.sshauthzservice),
                                 error => this.signingError(error,token.sshauthzservice));
 }

public getKeys(id?: Identity) {
  try{
    return JSON.parse(sessionStorage.getItem('keys'));
  } catch {
    return [];
  }
}

 makeKeyCert(key: string, resp, sshauthzservice: SshAuthzServer) {
   let keyCert = new KeyCert()
   keyCert.key = key;
   keyCert.cert = resp['cert'];
   var keys: KeyCert[] = [];
   try{
     keys = JSON.parse(sessionStorage.getItem('keys'));
   } catch {
     keys = [];
   }
   if (keys === null) {
     keys = [];
   }
   keys.push(keyCert);
   sessionStorage.setItem('keys',JSON.stringify(keys))
   this.sshAdd(keyCert);
   // this.tesService.keyCert.next(keyCert);
   // As soon as the certificate has been generated, we log back out of the signing server
   if (!(sshauthzservice.logout === null)) {
     window.open(sshauthzservice.logout);
   }
   let path=localStorage.getItem('path');
   this.readyToNavigate.next([true,path]);
 }

 public querySshAgentError(error: any) {
   this.agentContents.next([]);
   if (error.status == 0) {
     this.statusMsg.next("A network error occured. Are you connected to the internet?")
   }
   this.statusMsg.next("Error querying ssh agent");
 }

 public updateAgentContents() {
   if (this.backendURI == null) {
     return
   }
   console.log('attempting to update agent contents');
   console.log(this.backendSelectionService.apiserver)
   console.log(this.backendURI);
   if (this.statusMsg !== undefined) {
    this.statusMsg.next("Updating the list of available accounts");
   };
   let headers = new HttpHeaders();
   let options = { headers: headers, withCredentials: true};
   var anyvar: any;
   this.http.get<any>(this.backendURI+'/sshagent',options)
                     .pipe(catchError(this.handleError(anyvar)))
                     .subscribe(resp => { this.agentContents.next(resp); this.statusMsg.next("") },
                                error => this.querySshAgentError(error));
                     // .subscribe(resp => this.computeSitesService.updateIdentities(resp),
                     //            error => this.httperror(error))
 }


 private killAgent() {
   this.statusMsg.next("Logging out")
   let headers = new HttpHeaders();
   let options = { headers: headers, withCredentials: true};
   var anyvar: any;
   this.http.delete<any>(this.backendURI+'/sshagent',options)
                     .pipe(catchError(this.handleError(anyvar)))
                     .subscribe(resp => this.updateAgentContents(),
                                error => this.httperror(error));
 }

 public logout(): Boolean {
   this.killAgent();
   return true;
 }



 public sshAdd(keyCert: KeyCert) {
   if (keyCert.key == undefined) {
     return;
   }
   let headers = new HttpHeaders();
   let options = { headers: headers, withCredentials: true};
   var anyvar: any;

   this.statusMsg.next("Authorising ...")
   let data = {'key': keyCert.key, 'cert': keyCert.cert};
   this.http.post<any>(this.backendURI+'/sshagent',data,options)
                       .pipe(catchError(this.handleError(anyvar)))
                       .subscribe(resp => this.updateAgentContents(),
                                  error => this.httperror(error))
 }

 public login(authservice: SshAuthzServer) {
   let redirect_uri = window.location.origin+this.locationStrategy.getBaseHref()+"sshauthz_callback";
   let nonce=Math.random().toString(36).substring(2, 15)



   localStorage.setItem('authservice', JSON.stringify([authservice,nonce]));
   localStorage.setItem('path', this.location.path());
   if (authservice.scope == null) {
     window.location.assign(authservice.authorise+"?response_type=token&redirect_uri="+redirect_uri+"&state="+nonce+"&client_id="+authservice.client_id);
   } else {
     window.location.assign(authservice.authorise+"?response_type=token&redirect_uri="+redirect_uri+"&state="+nonce+"&client_id="+authservice.client_id+"&scope="+authservice.scope);
   }
 }

 // private handleError(error: HttpErrorResponse) {
 //   if (error.error instanceof ErrorEvent) {
 //     console.log('instance of ErrorEvent');
 //     console.log(error);
 //     // A client-side or network error occurred. Handle it accordingly.
 //     return throwError("Hmm, that didn't work. If you're using a local connection, please make sure Strudel-TES is running.");
 //   } else {
 //     console.log('NOT an instance of ErrorEvent');
 //     console.log(error);
 //     // Not sure if this code works correctly. It should update identities in case the error is
 //     // that the user isn't allowed to run the job
 //
 //     var re = /identity/gi;
 //     let searchresult = error.error.message.search(re);
 //     if (searchresult != -1) {
 //       // this.getIdentities();
 //       return throwError('login expired, refreshing');
 //     }
 //     return throwError(error.error.message)
 //     // this.statusMsg.next("There was an error submitting that job. The backend gave me the message: " + error.error.message);
 //
 //   }

 private httperror(error: any) {
   this.statusMsg.next('There was an error logging in or generating crypto tokens');
   console.error(error);
 }
 private signingError(error: any,sshauthzservice: SshAuthzServer) {
   this.statusMsg.next('You don\'t appear to have an account on '+sshauthzservice.name);
   if (!(sshauthzservice.logout === null)) {
     window.open(sshauthzservice.logout);
   }
   let path=localStorage.getItem('path');
   this.readyToNavigate.next([true,path]);
 }
 

 private handleError<T> (result?: T) {
   return (error: any): Observable<T> => {
     if (error.status == 500) {
       return throwError("The backend server encountered and error. Please try again in a few minutes")
     }
     return throwError(error.message);
     // return of(result as T);
   };
 }
}
