import { Injectable } from '@angular/core';
import { Computesite, Strudelapp } from './computesite';
import {BehaviorSubject} from 'rxjs/BehaviorSubject';
import { HttpClientModule, HttpClient, HttpHeaders, HttpErrorResponse } from '@angular/common/http';
import { Observable } from 'rxjs/Observable';
import { of } from 'rxjs/observable/of';
import { catchError, map, tap } from 'rxjs/operators';
import { Identity } from './identity';
import { TesService } from './tes.service';
import { AuthorisationService} from './authorisation.service';



@Injectable({
  providedIn: 'root',
})
export class ComputesitesService {
  public computesites: BehaviorSubject<Computesite[]>;
  public identities: BehaviorSubject<Identity[]>;
  public appidentities: BehaviorSubject<Identity[]>;
  public ftidentities: BehaviorSubject<Identity[]>;
  private statusMsg: BehaviorSubject<any>;

  constructor(private http: HttpClient, private authorisationService: AuthorisationService) {
    this.computesites = new BehaviorSubject<Computesite[]>([]);
    this.identities = new BehaviorSubject<Identity[]>([]);
    this.appidentities = new BehaviorSubject<Identity[]>([]);
    this.ftidentities = new BehaviorSubject<Identity[]>([]);

    this.computesites.subscribe(computesites => this.getStrudelApps(computesites))
    this.computesites.subscribe((cs) => this.authorisationService.updateAgentContents());
    this.authorisationService.agentContents.subscribe(ac => this.updateIdentities(ac));
    this.getComputeSites();
  }

  public setStatusMsg(statusMsg: BehaviorSubject<any>) {
    this.statusMsg = statusMsg;
  }

  getStrudelApps(computesites: Computesite[]) {
    for (let s of computesites) {
      let headers = new HttpHeaders();
      let options = { headers: headers, withCredentials: false};
      if (s.appCatalogUri !== null) {
        this.http.get<Strudelapp[]>(s.appCatalogUri,options)
                         .pipe(catchError(this.handleError('getStrudelApps')))
                         .subscribe(resp => this.updateStrudelApps(s,resp));
      }
    }

  }

  storeLocalStrudelApps(name: string, apps: any) {
      localStorage.setItem(name+'-apps',JSON.stringify(apps))
  }

  removeLocalStrudelApps() {
      var keys = [];
      for (var key in localStorage) {
        var idx = key.indexOf('-apps');
        if (idx !== -1) {
            keys.push(key)
        }
      }
      for (var key in keys) {
          localStorage.removeItem(key);
      }
  }


  updateStrudelApps(site: Computesite, apps) {
    var sapps: Strudelapp[];
    var localapps: Strudelapp[];
    sapps = <Strudelapp[]>apps;

    localapps = JSON.parse(localStorage.getItem(site.name+'-apps'))
    if (localapps !== null)  {
      for (let a of localapps) {
        sapps.push(a);
      }
    }
    site.appCatalog.next(sapps);
  }

  private handleError<T> (operation = 'operation', result?: T) {
    return (error: any): Observable<T> => {
      console.error(error);
      return of(result as T);
    };
  }


  getComputeSites() {
    let headers = new HttpHeaders();
    let options = { headers: headers, withCredentials: false};
    this.http.get<Strudelapp[]>('./assets/config/computesites.json',options)
                     .pipe(catchError(this.handleError('getComputeSites')))
                     .subscribe(resp => this.updateComputeSites(resp));
  }

  storeLocalComputeSites(computesites: any) {
      try {
        localStorage.setItem('localcomputesites',JSON.stringify(computesites));
      } catch {
      }
      this.getComputeSites();
  }

  removeLocalComputeSites() {
      localStorage.removeItem('localcomputesites');
      this.getComputeSites();
  }

  updateComputeSites(resp) {
    var localcomputesites: Computesite[] = [];
    var computesites: Computesite[] = []
    try {
        localcomputesites = JSON.parse(localStorage.getItem('localcomputesites'))
    } catch {
        localcomputesites = []
    }
    if (localcomputesites !== null ) {
      for (let cs of localcomputesites) {
        cs.appCatalog  = new BehaviorSubject<Strudelapp[]>([]);
        computesites.push(cs);
      }
    }
    for (let cs of resp) {
      let computesite = <Computesite>cs;
      computesite.appCatalog = new BehaviorSubject<Strudelapp[]>([])
      computesites.push(computesite);
    }
    this.computesites.next(computesites);
  }

  returnComputeSites(): Computesite[] {
    return this.computesites.value;
  }

  private siteMatch(cert: any, cs: Computesite): string {
    var fp: string = (<Computesite>cs).cafingerprint;
    if ('Signing CA' in cert) {
      for (let ca of cert['Signing CA']) {
        if (ca == fp) {
          return cert['Principals'][0]
        }
      }
    }
    return null;
  }

  private updateIdentities(resp) {
    let certs = resp;
    var cs: Computesite;
    var identities: Identity[] = [];
    var appidentities: Identity[] = [];
    var ftidentities: Identity[] = [];
    for (cs of this.computesites.value) {
      for (let i in certs) {
        let principal = this.siteMatch(certs[i],cs);
        if (principal != null) {
            let id = new Identity(principal,cs);
            identities.push(id);
            if (cs.appCatalogUri != null) {
              appidentities.push(id);
            }
            if (cs.dtn != null ) {
              ftidentities.push(id);
            }
          }
        }
      }
      this.identities.next(identities);
      this.ftidentities.next(ftidentities);
      this.appidentities.next(appidentities);

    }

}
