import { Component, OnInit } from '@angular/core';
import {BackendSelectionService } from '../backend-selection.service';
import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import {OverlayContainer} from '@angular/cdk/overlay';
import { AuthorisationService } from '../authorisation.service';
import { ComputesitesService } from '../computesites.service';

@Component({
  selector: 'app-settings',
  templateUrl: './settings.component.html',
  styleUrls: ['./settings.component.css']
})
export class SettingsComponent implements OnInit {

  public selectedApiServer: any;
  private file: any;
  private config: any;
  public theme: any;
  public themeSubject: BehaviorSubject<string>;
  public themes: any[] = [ {'name':'Light','value':'strudel-light-theme'},
                   {'name': 'Dark','value':'strudel-dark-theme'}]
  constructor(
                public backendSelectionService: BackendSelectionService,
                public authService: AuthorisationService,
                public computeSitesService: ComputesitesService,
                public overlayContainer: OverlayContainer,
    ) { 
      this.backendSelectionService.apiserver.subscribe((s) => this.selectedApiServer = s);
  }

  ngOnInit() {
    this.setTheme();
  }

  loadConfig(event) {
      const reader = new FileReader();
      reader.onload = (e: any) => {
          try {
            this.config = JSON.parse(e.target.result);
              
              if ('computesites' in this.config) {
                  this.computeSitesService.storeLocalComputeSites(this.config['computesites'])
              }
              if ('apps' in this.config) {
                  this.computeSitesService.storeLocalStrudelApps(this.config['sitename'],this.config['apps']);
              }
              if ('authz' in this.config) {
                  this.authService.storeLocalAuthZ(this.config['authz'])
              }

          } catch {
              console.log('local config file is invalid')
          }
      };
      reader.readAsText(event.target.files[0]);
  }


  resetConfig() {
      this.computeSitesService.removeLocalStrudelApps();
      this.computeSitesService.removeLocalComputeSites();
      this.authService.removeLocalAuthZ()
  }

  setTheme()  {
    var theme;
    theme=localStorage.getItem('strudel-theme');
    if (theme == null) {
      theme = 'strudel-theme-light';
    }
    let classList = document.querySelector('app-root').classList
    var c;
    for (c of classList.value.split(' ')) {
      if (c.indexOf('strudel-') == 0) {
        console.log('remove class'+c);
        classList.remove(c);
      }
    }
    classList.add(theme)
}

  selectTheme(event) {
    localStorage.setItem('strudel-theme',event.value);
    this.setTheme();
  }

}
